'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var useSafeCallback = _interopDefault(require('use-safe-callback'));

// A type of promise-like that resolves synchronously and supports only one observer

const _iteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.iterator || (Symbol.iterator = Symbol("Symbol.iterator"))) : "@@iterator";

const _asyncIteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.asyncIterator || (Symbol.asyncIterator = Symbol("Symbol.asyncIterator"))) : "@@asyncIterator";

// Asynchronously call a function and send errors to recovery continuation
function _catch(body, recover) {
	try {
		var result = body();
	} catch(e) {
		return recover(e);
	}
	if (result && result.then) {
		return result.then(void 0, recover);
	}
	return result;
}

function noop() {}
/**
 * Get the latest value received as parameter, useful to be able to dynamically
 * read a value from params inside a callback or effect without cleaning and
 * running again the effect or recreating the callback.
 */


function useGetLatest(value) {
  var ref = React.useRef(value);
  ref.current = value;
  return React.useCallback(function () {
    return ref.current;
  }, []);
}
/**
 * Hook to run async function which cause a side-effect, specially useful to
 * run requests against an API
 */


function useMutation(mutationFn, _temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      _ref$onMutate = _ref.onMutate,
      onMutate = _ref$onMutate === void 0 ? function () {
    return noop;
  } : _ref$onMutate,
      _ref$onSuccess = _ref.onSuccess,
      onSuccess = _ref$onSuccess === void 0 ? noop : _ref$onSuccess,
      _ref$onFailure = _ref.onFailure,
      onFailure = _ref$onFailure === void 0 ? noop : _ref$onFailure,
      _ref$onSettled = _ref.onSettled,
      onSettled = _ref$onSettled === void 0 ? noop : _ref$onSettled,
      _ref$throwOnFailure = _ref.throwOnFailure,
      throwOnFailure = _ref$throwOnFailure === void 0 ? false : _ref$throwOnFailure,
      _ref$useErrorBoundary = _ref.useErrorBoundary,
      useErrorBoundary = _ref$useErrorBoundary === void 0 ? false : _ref$useErrorBoundary;

  var _React$useReducer = React.useReducer(function reducer(_, action) {
    if (action.type === 'RESET') {
      return {
        status: 'idle'
      };
    }

    if (action.type === 'MUTATE') {
      return {
        status: 'running'
      };
    }

    if (action.type === 'SUCCESS') {
      return {
        status: 'success',
        data: action.data
      };
    }

    if (action.type === 'FAILURE') {
      return {
        status: 'failure',
        error: action.error
      };
    }

    throw Error('Invalid action');
  }, {
    status: 'idle'
  }),
      _React$useReducer$ = _React$useReducer[0],
      status = _React$useReducer$.status,
      data = _React$useReducer$.data,
      error = _React$useReducer$.error,
      unsafeDispatch = _React$useReducer[1];

  var getMutationFn = useGetLatest(mutationFn);
  var latestMutation = React.useRef(0);
  var safeDispatch = useSafeCallback(unsafeDispatch);
  /**
   * Run your mutation function, this function receives an input value and pass
   * it directly to your mutation function.
   */

  var mutate = React.useCallback(function mutate(input, config) {
    if (config === void 0) {
      config = {};
    }

    try {
      var mutation = Date.now();
      latestMutation.current = mutation;
      safeDispatch({
        type: 'MUTATE'
      });
      return Promise.resolve(onMutate({
        input: input
      })).then(function (rollback) {
        return _catch(function () {
          return Promise.resolve(getMutationFn()(input)).then(function (data) {
            if (latestMutation.current === mutation) {
              safeDispatch({
                type: 'SUCCESS',
                data: data
              });
            }

            return Promise.resolve(onSuccess({
              data: data,
              input: input
            })).then(function () {
              var _config$onSuccess;

              return Promise.resolve(((_config$onSuccess = config.onSuccess) !== null && _config$onSuccess !== void 0 ? _config$onSuccess : noop)({
                data: data,
                input: input
              })).then(function () {
                return Promise.resolve(onSettled({
                  status: 'success',
                  data: data,
                  input: input
                })).then(function () {
                  var _config$onSettled;

                  return Promise.resolve(((_config$onSettled = config.onSettled) !== null && _config$onSettled !== void 0 ? _config$onSettled : noop)({
                    status: 'success',
                    data: data,
                    input: input
                  })).then(function () {
                    return data;
                  });
                });
              });
            });
          });
        }, function (error) {
          return Promise.resolve(onFailure({
            error: error,
            rollback: rollback,
            input: input
          })).then(function () {
            var _config$onFailure;

            return Promise.resolve(((_config$onFailure = config.onFailure) !== null && _config$onFailure !== void 0 ? _config$onFailure : noop)({
              error: error,
              rollback: rollback,
              input: input
            })).then(function () {
              return Promise.resolve(onSettled({
                status: 'failure',
                error: error,
                input: input,
                rollback: rollback
              })).then(function () {
                var _config$onSettled2;

                return Promise.resolve(((_config$onSettled2 = config.onSettled) !== null && _config$onSettled2 !== void 0 ? _config$onSettled2 : noop)({
                  status: 'failure',
                  error: error,
                  input: input,
                  rollback: rollback
                })).then(function () {
                  var _config$throwOnFailur;

                  if (latestMutation.current === mutation) {
                    safeDispatch({
                      type: 'FAILURE',
                      error: error
                    });
                  }

                  if ((_config$throwOnFailur = config.throwOnFailure) !== null && _config$throwOnFailur !== void 0 ? _config$throwOnFailur : throwOnFailure) throw error;
                });
              });
            });
          });
        });
      });
    } catch (e) {
      return Promise.reject(e);
    }
  }, []);
  var reset = React.useCallback(function reset() {
    safeDispatch({
      type: 'RESET'
    });
  }, []);
  if (useErrorBoundary && error) throw error;
  return [mutate, {
    status: status,
    data: data,
    error: error,
    reset: reset
  }];
}

exports.default = useMutation;
//# sourceMappingURL=use-mutation.cjs.development.js.map
